@@ -26,29 +26,36 @@
 
 #include <glib.h>
 #include <gtk/gtk.h>
-#include <exo/exo.h>
+//#include <exo/exo.h>
 #include <libxfce4panel/libxfce4panel.h>
 #include <libindicator/indicator-object.h>
 
 #include "indicator-box.h"
 #include "indicator-button.h"
 
-static void                 xfce_indicator_box_finalize       (GObject          *object);
-static void                 xfce_indicator_box_list_changed   (XfceIndicatorBox *box,
-                                                               IndicatorConfig  *config);
-static void                 xfce_indicator_box_add            (GtkContainer     *container,
-                                                               GtkWidget        *child);
-static void                 xfce_indicator_box_remove         (GtkContainer     *container,
-                                                               GtkWidget        *child);
-static void                 xfce_indicator_box_forall         (GtkContainer     *container,
-                                                               gboolean          include_internals,
-                                                               GtkCallback       callback,
-                                                               gpointer          callback_data);
-static GType                xfce_indicator_box_child_type     (GtkContainer     *container);
-static void                 xfce_indicator_box_size_request   (GtkWidget        *widget,
-                                                               GtkRequisition   *requisition);
-static void                 xfce_indicator_box_size_allocate  (GtkWidget        *widget,
-                                                               GtkAllocation    *allocation);
+static void                 xfce_indicator_box_finalize             (GObject          *object);
+static void                 xfce_indicator_box_list_changed         (XfceIndicatorBox *box,
+                                                                     IndicatorConfig  *config);
+static void                 xfce_indicator_box_add                  (GtkContainer     *container,
+                                                                     GtkWidget        *child);
+static void                 xfce_indicator_box_remove               (GtkContainer     *container,
+                                                                     GtkWidget        *child);
+static void                 xfce_indicator_box_forall               (GtkContainer     *container,
+                                                                     gboolean          include_internals,
+                                                                     GtkCallback       callback,
+                                                                     gpointer          callback_data);
+static GType                xfce_indicator_box_child_type           (GtkContainer     *container);
+static void                 xfce_indicator_box_get_preferred_length (GtkWidget        *widget,
+                                                                     gint             *minimal_length,
+                                                                     gint             *natural_length);
+static void                 xfce_indicator_box_get_preferred_width  (GtkWidget        *widget,
+                                                                     gint             *minimal_width,
+                                                                     gint             *natural_width);
+static void                 xfce_indicator_box_get_preferred_height (GtkWidget        *widget,
+                                                                     gint             *minimal_height,
+                                                                     gint             *natural_height);
+static void                 xfce_indicator_box_size_allocate        (GtkWidget        *widget,
+                                                                     GtkAllocation    *allocation);
 
 
 struct _XfceIndicatorBox
@@ -59,14 +66,6 @@ struct _XfceIndicatorBox
 
   GHashTable           *children;
 
-  gint                  panel_size;
-  gint                  nrows;
-  gint                  icon_size_max;
-  gboolean              align_left;
-
-  GtkOrientation        panel_orientation;
-  GtkOrientation        orientation;
-
   gulong                indicator_list_changed_id;
 };
 
@@ -91,7 +90,9 @@ xfce_indicator_box_class_init (XfceIndicatorBoxClass *klass)
   gobject_class->finalize = xfce_indicator_box_finalize;
 
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
-  gtkwidget_class->size_request = xfce_indicator_box_size_request;
+  //gtkwidget_class->size_request = xfce_indicator_box_size_request;
+  gtkwidget_class->get_preferred_width = xfce_indicator_box_get_preferred_width;
+  gtkwidget_class->get_preferred_height = xfce_indicator_box_get_preferred_height;
   gtkwidget_class->size_allocate = xfce_indicator_box_size_allocate;
 
   gtkcontainer_class = GTK_CONTAINER_CLASS (klass);
@@ -106,8 +107,7 @@ xfce_indicator_box_class_init (XfceIndicatorBoxClass *klass)
 static void
 xfce_indicator_box_init (XfceIndicatorBox *box)
 {
-  GTK_WIDGET_SET_FLAGS (box, GTK_NO_WINDOW);
-
+  gtk_widget_set_has_window (GTK_WIDGET (box), FALSE);
   gtk_widget_set_can_focus(GTK_WIDGET(box), TRUE);
   gtk_container_set_border_width(GTK_CONTAINER(box), 0);
 
@@ -217,7 +217,7 @@ xfce_indicator_box_add (GtkContainer *container,
 
   g_return_if_fail (XFCE_IS_INDICATOR_BOX (box));
   g_return_if_fail (XFCE_IS_INDICATOR_BUTTON (button));
-  g_return_if_fail (child->parent == NULL);
+  g_return_if_fail (gtk_widget_get_parent (GTK_WIDGET (child)) == NULL);
 
   io_name = xfce_indicator_button_get_io_name (button);
   li = g_hash_table_lookup (box->children, io_name);
@@ -306,17 +306,9 @@ xfce_indicator_box_child_type (GtkContainer *container)
 static gint
 xfce_indicator_box_get_row_size (XfceIndicatorBox *box)
 {
-  gint                 border_thickness;
-  GtkStyle            *style;
-
-  g_return_val_if_fail (XFCE_IS_INDICATOR_BOX (box), 24);
-
-  style = gtk_widget_get_style (GTK_WIDGET (box));
-  border_thickness = 2 * MAX (style->xthickness, style->ythickness) + 2;
-
   return MIN (indicator_config_get_panel_size (box->config) /
               indicator_config_get_nrows (box->config),
-              indicator_config_get_icon_size_max (box->config) + border_thickness);
+              indicator_config_get_row_size_max (box->config));
 }
 
 
@@ -324,8 +316,9 @@ xfce_indicator_box_get_row_size (XfceIndicatorBox *box)
 
 
 static void
-xfce_indicator_box_size_request (GtkWidget      *widget,
-                                 GtkRequisition *requisition)
+xfce_indicator_box_get_preferred_length (GtkWidget *widget,
+                                         gint      *minimum_length,
+                                         gint      *natural_length)
 {
   XfceIndicatorBox    *box = XFCE_INDICATOR_BOX (widget);
   XfceIndicatorButton *button;
@@ -357,7 +350,7 @@ xfce_indicator_box_size_request (GtkWidget      *widget,
         {
           button = XFCE_INDICATOR_BUTTON (li_tmp->data);
 
-          gtk_widget_size_request (GTK_WIDGET (button), &child_req);
+          gtk_widget_get_preferred_size (GTK_WIDGET (button), NULL, &child_req);
           has_label = (xfce_indicator_button_get_label (button) != NULL);
           rectangular_icon = xfce_indicator_button_is_icon_rectangular (button);
 
@@ -387,21 +380,66 @@ xfce_indicator_box_size_request (GtkWidget      *widget,
 
   x += length;
 
-  if (panel_orientation == GTK_ORIENTATION_HORIZONTAL)
+  if (minimum_length != NULL)
+    *minimum_length = x;
+
+  if (natural_length != NULL)
+    *natural_length = x;
+
+  /* g_debug ("indicator-box size request: %d", x); */
+}
+
+
+
+static void
+xfce_indicator_box_get_preferred_width (GtkWidget *widget,
+                                        gint      *minimum_width,
+                                        gint      *natural_width)
+{
+  XfceIndicatorBox *box = XFCE_INDICATOR_BOX (widget);
+  gint              panel_size;
+
+  if (indicator_config_get_panel_orientation (box->config) == GTK_ORIENTATION_HORIZONTAL)
     {
-      requisition->width = x;
-      requisition->height = panel_size;
+      xfce_indicator_box_get_preferred_length (widget, minimum_width, natural_width);
     }
   else
     {
-      requisition->width = panel_size;
-      requisition->height = x;
+      panel_size = indicator_config_get_panel_size (box->config);
+      if (minimum_width != NULL)
+        *minimum_width = panel_size;
+      if (natural_width != NULL)
+        *natural_width = panel_size;
     }
-  /* g_debug ("indicator-box size request: w=%d h=%d", requisition->width, requisition->height); */
 }
 
 
 
+static void
+xfce_indicator_box_get_preferred_height (GtkWidget *widget,
+                                         gint      *minimum_height,
+                                         gint      *natural_height)
+{
+  XfceIndicatorBox *box = XFCE_INDICATOR_BOX (widget);
+  gint              panel_size;
+
+  if (indicator_config_get_panel_orientation (box->config) == GTK_ORIENTATION_VERTICAL)
+    {
+      xfce_indicator_box_get_preferred_length (widget, minimum_height, natural_height);
+    }
+  else
+    {
+      panel_size = indicator_config_get_panel_size (box->config);
+      if (minimum_height != NULL)
+        *minimum_height = panel_size;
+      if (natural_height != NULL)
+        *natural_height = panel_size;
+    }
+}
+
+
+
+
 static void
 xfce_indicator_box_size_allocate (GtkWidget     *widget,
                                   GtkAllocation *allocation)
@@ -427,6 +465,8 @@ xfce_indicator_box_size_allocate (GtkWidget     *widget,
   x0 = allocation->x;
   y0 = allocation->y;
 
+  gtk_widget_set_allocation (widget, allocation);
+
   panel_size = indicator_config_get_panel_size (box->config);
   panel_orientation = indicator_config_get_panel_orientation (box->config);
   nrows = panel_size / xfce_indicator_box_get_row_size (box);
@@ -443,7 +483,7 @@ xfce_indicator_box_size_allocate (GtkWidget     *widget,
         {
           button = XFCE_INDICATOR_BUTTON (li_tmp->data);
 
-          gtk_widget_get_child_requisition (GTK_WIDGET (button), &child_req);
+          gtk_widget_get_preferred_size (GTK_WIDGET (button), NULL, &child_req);
 
           has_label = (xfce_indicator_button_get_label (button) != NULL);
           rectangular_icon = xfce_indicator_button_is_icon_rectangular (button);
