@@ -36,7 +36,7 @@
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
 #include <xfconf/xfconf.h>
-#include <exo/exo.h>
+//#include <exo/exo.h>
 #include <libxfce4panel/xfce-panel-plugin.h>
 
 #include "indicator.h"
@@ -45,7 +45,7 @@
 
 
 
-#define DEFAULT_ICON_SIZE_MAX      22
+#define DEFAULT_ROW_SIZE_MAX       24
 #define DEFAULT_ALIGN_LEFT         FALSE
 #define DEFAULT_EXCLUDED_MODULES   NULL
 #define DEFAULT_ORIENTATION        GTK_ORIENTATION_HORIZONTAL
@@ -77,7 +77,7 @@ struct _IndicatorConfig
 {
   GObject          __parent__;
 
-  gint             icon_size_max;
+  gint             row_size_max;
   gboolean         align_left;
   gboolean         mode_whitelist;
   GHashTable      *blacklist;
@@ -98,7 +98,7 @@ struct _IndicatorConfig
 enum
 {
   PROP_0,
-  PROP_ICON_SIZE_MAX,
+  PROP_ROW_SIZE_MAX,
   PROP_ALIGN_LEFT,
   PROP_MODE_WHITELIST,
   PROP_BLACKLIST,
@@ -150,38 +150,44 @@ indicator_config_class_init (IndicatorConfigClass *klass)
   gobject_class->set_property = indicator_config_set_property;
 
   g_object_class_install_property (gobject_class,
-                                   PROP_ICON_SIZE_MAX,
-                                   g_param_spec_uint ("icon-size-max",
+                                   PROP_ROW_SIZE_MAX,
+                                   g_param_spec_uint ("row-size-max",
                                                       NULL, NULL,
                                                       1,
                                                       128,
-                                                      DEFAULT_ICON_SIZE_MAX,
-                                                      EXO_PARAM_READWRITE));
+                                                      DEFAULT_ROW_SIZE_MAX,
+                                                      G_PARAM_READWRITE |
+                                                      G_PARAM_STATIC_STRINGS));
+
   g_object_class_install_property (gobject_class,
                                    PROP_ALIGN_LEFT,
                                    g_param_spec_boolean ("align-left", NULL, NULL,
                                                          DEFAULT_ALIGN_LEFT,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE |
+                                                         G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_MODE_WHITELIST,
                                    g_param_spec_boolean ("mode-whitelist", NULL, NULL,
                                                          DEFAULT_MODE_WHITELIST,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE |
+                                                         G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_BLACKLIST,
                                    g_param_spec_boxed ("blacklist",
                                                        NULL, NULL,
                                                        XFCE_TYPE_INDICATOR_CONFIG_VALUE_ARRAY,
-                                                       EXO_PARAM_READWRITE));
+                                                       G_PARAM_READWRITE |
+                                                       G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_WHITELIST,
                                    g_param_spec_boxed ("whitelist",
                                                        NULL, NULL,
                                                        XFCE_TYPE_INDICATOR_CONFIG_VALUE_ARRAY,
-                                                       EXO_PARAM_READWRITE));
+                                                       G_PARAM_READWRITE |
+                                                       G_PARAM_STATIC_STRINGS));
 
 
   g_object_class_install_property (gobject_class,
@@ -189,7 +195,8 @@ indicator_config_class_init (IndicatorConfigClass *klass)
                                    g_param_spec_boxed ("known-indicators",
                                                        NULL, NULL,
                                                        XFCE_TYPE_INDICATOR_CONFIG_VALUE_ARRAY,
-                                                       EXO_PARAM_READWRITE));
+                                                       G_PARAM_READWRITE |
+                                                       G_PARAM_STATIC_STRINGS));
 
 
   indicator_config_signals[CONFIGURATION_CHANGED] =
@@ -214,7 +221,7 @@ indicator_config_class_init (IndicatorConfigClass *klass)
 static void
 indicator_config_init (IndicatorConfig *config)
 {
-  config->icon_size_max        = DEFAULT_ICON_SIZE_MAX;
+  config->row_size_max         = DEFAULT_ROW_SIZE_MAX;
   config->align_left           = DEFAULT_ALIGN_LEFT;
   config->mode_whitelist       = DEFAULT_MODE_WHITELIST;
   config->blacklist            = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
@@ -274,8 +281,8 @@ indicator_config_get_property (GObject    *object,
 
   switch (prop_id)
     {
-    case PROP_ICON_SIZE_MAX:
-      g_value_set_uint (value, config->icon_size_max);
+    case PROP_ROW_SIZE_MAX:
+      g_value_set_uint (value, config->row_size_max);
       break;
 
     case PROP_ALIGN_LEFT:
@@ -336,11 +343,11 @@ indicator_config_set_property (GObject      *object,
 
   switch (prop_id)
     {
-    case PROP_ICON_SIZE_MAX:
+    case PROP_ROW_SIZE_MAX:
       val = g_value_get_uint (value);
-      if (config->icon_size_max != val)
+      if (config->row_size_max != val)
         {
-          config->icon_size_max = val;
+          config->row_size_max = val;
           g_signal_emit (G_OBJECT (config), indicator_config_signals [CONFIGURATION_CHANGED], 0);
         }
       break;
@@ -423,11 +430,11 @@ indicator_config_set_property (GObject      *object,
 
 
 gint
-indicator_config_get_icon_size_max (IndicatorConfig *config)
+indicator_config_get_row_size_max (IndicatorConfig *config)
 {
-  g_return_val_if_fail (XFCE_IS_INDICATOR_CONFIG (config), DEFAULT_ICON_SIZE_MAX);
+  g_return_val_if_fail (XFCE_IS_INDICATOR_CONFIG (config), DEFAULT_ROW_SIZE_MAX);
 
-  return config->icon_size_max;
+  return config->row_size_max;
 }
 
 
@@ -584,7 +591,7 @@ indicator_config_blacklist_set (IndicatorConfig *config,
   gchar *name_copy;
 
   g_return_if_fail (XFCE_IS_INDICATOR_CONFIG (config));
-  g_return_if_fail (!exo_str_is_empty (name));
+  //g_return_if_fail (!exo_str_is_empty (name));
 
   if (add)
     {
@@ -611,7 +618,7 @@ indicator_config_whitelist_set (IndicatorConfig *config,
   gchar *name_copy;
 
   g_return_if_fail (XFCE_IS_INDICATOR_CONFIG (config));
-  g_return_if_fail (!exo_str_is_empty (name));
+  //g_return_if_fail (!exo_str_is_empty (name));
 
   if (add)
     {
@@ -660,7 +667,7 @@ indicator_config_add_known_indicator (IndicatorConfig *config,
   GList    *li;
 
   g_return_if_fail (XFCE_IS_INDICATOR_CONFIG (config));
-  g_return_if_fail (!exo_str_is_empty (name));
+  //g_return_if_fail (!exo_str_is_empty (name));
 
   /* check if the indicator is already known */
   for(li = config->known_indicators; li != NULL; li = li->next)
@@ -684,8 +691,8 @@ indicator_config_swap_known_indicators (IndicatorConfig *config,
   GList       *li, *li_tmp;
 
   g_return_if_fail (XFCE_IS_INDICATOR_CONFIG (config));
-  g_return_if_fail (!exo_str_is_empty (name1));
-  g_return_if_fail (!exo_str_is_empty (name2));
+  //g_return_if_fail (!exo_str_is_empty (name1));
+  //g_return_if_fail (!exo_str_is_empty (name2));
 
   for(li = config->known_indicators; li != NULL; li = li->next)
     if (g_strcmp0 (li->data, name1) == 0)
@@ -750,8 +757,8 @@ indicator_config_new (const gchar     *property_base)
     {
       channel = xfconf_channel_get ("xfce4-panel");
 
-      property = g_strconcat (property_base, "/icon-size-max", NULL);
-      xfconf_g_property_bind (channel, property, G_TYPE_INT, config, "icon-size-max");
+      property = g_strconcat (property_base, "/row-size-max", NULL);
+      xfconf_g_property_bind (channel, property, G_TYPE_INT, config, "row-size-max");
       g_free (property);
 
       property = g_strconcat (property_base, "/align-left", NULL);
