@@ -35,7 +35,7 @@
 
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
-#include <exo/exo.h>
+//#include <exo/exo.h>
 #include <libxfce4panel/xfce-panel-plugin.h>
 
 #include "indicator-dialog.h"
@@ -60,8 +60,11 @@ static const gchar *pretty_names[][3] =
   { "libpower.so",              N_("Power Management"),                     NULL },
   { "libappmenu.so",            N_("Application Menus (Global Menu)"),      "menu-editor" },
   { "libsession.so",            N_("Session Management"),                   NULL },
-  { "libmessaging.so",          N_("Messaging Menu"),                       "indicator-messages" },
+  { "libmessaging.so",          N_("Messaging Menu"),                       "indicator-messages-new" },
   { "libdatetime.so",           N_("Date and Time"),                        "time-admin" },
+  { "libbluetooth.so",          N_("Bluetooth"),                            "bluetooth-active" },
+  { "libsyncindicator.so",      N_("Sync Menu"),                            "sync-client-updating" },
+  { "libworkrave.so",           N_("Workrave"),                             "workrave" },
 };
 
 
@@ -171,6 +174,8 @@ indicator_dialog_update_indicator_names (IndicatorDialog *dialog)
       name = li->data;
 
       /* check if we have a better name for the application */
+      pretty_name = NULL;
+      icon_name = NULL;
       for (i = 0; i < G_N_ELEMENTS (pretty_names); i++)
         {
           if (strcmp (name, pretty_names[i][0]) == 0)
@@ -478,18 +483,27 @@ indicator_dialog_build (IndicatorDialog *dialog)
 
       object = gtk_builder_get_object (builder, "size-max");
       g_return_if_fail (GTK_IS_WIDGET (object));
-      exo_mutual_binding_new (G_OBJECT (dialog->config), "icon-size-max",
-                              G_OBJECT (object), "value");
+      //exo_mutual_binding_new (G_OBJECT (dialog->config), "row-size-max",
+      //                        G_OBJECT (object), "value");
+      g_object_bind_property (G_OBJECT (dialog->config), "row-size-max",
+                              G_OBJECT (object), "value",
+                              G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
       object = gtk_builder_get_object (builder, "checkbutton-align-left");
       g_return_if_fail (GTK_IS_WIDGET (object));
-      exo_mutual_binding_new (G_OBJECT (dialog->config), "align-left",
-                              G_OBJECT (object), "active");
+      //exo_mutual_binding_new (G_OBJECT (dialog->config), "align-left",
+      //                        G_OBJECT (object), "active");
+      g_object_bind_property (G_OBJECT (dialog->config), "align-left",
+                              G_OBJECT (object), "active",
+                              G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
       object = gtk_builder_get_object (builder, "checkbutton-whitelist");
       g_return_if_fail (GTK_IS_WIDGET (object));
-      exo_mutual_binding_new (G_OBJECT (dialog->config), "mode-whitelist",
-                              G_OBJECT (object), "active");
+      //exo_mutual_binding_new (G_OBJECT (dialog->config), "mode-whitelist",
+      //                        G_OBJECT (object), "active");
+      g_object_bind_property (G_OBJECT (dialog->config), "mode-whitelist",
+                              G_OBJECT (object), "active",
+                              G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
       g_signal_connect (G_OBJECT (object), "toggled",
                         G_CALLBACK (indicator_dialog_mode_whitelist_toggled), dialog);
       indicator_dialog_mode_whitelist_toggled (GTK_CHECK_BUTTON (object), dialog);
